/*  Copyright 2010-2011 Guillaume Duhamel

    This file is part of Gophrier.

    Gophrier is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Gophrier is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Gophrier; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
*/

#include "conf.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static char * confdir = NULL;
static char * rootdir = NULL;
static char * logdir = NULL;
static int isdaemon = -1;
static int useipv6 = -1;
static int port = -1;

static FILE * conf_open(const char * conf);
static int conf_read_int(const char * confkey, int defaultvalue);
static char * conf_read_string(const char * confkey, const char * defaultvalue);

char * conf_get_conf_dir()
{
    if (! confdir)
        return GOPHRIER_CONF_DIR;

    return confdir;
}

int conf_set_conf_dir(const char * path)
{
    confdir = strdup(path);
    return 0;
}

char * conf_get_root_dir()
{
    if (! rootdir)
        rootdir = conf_read_string("rootdir", GOPHRIER_DEFAULT_ROOT_DIR);

    return rootdir;
}

int conf_set_root_dir(const char * path)
{
    rootdir = strdup(path);
    return 0;
}

char * conf_get_log_dir()
{
    if (! logdir)
        logdir = conf_read_string("logdir", GOPHRIER_DEFAULT_LOG_DIR);

    return logdir;
}

int conf_set_log_dir(const char * path)
{
    logdir = strdup(path);
    return 0;
}

int conf_is_daemon()
{
    if (isdaemon == -1)
        isdaemon = conf_read_int("daemon", GOPHRIER_DEFAULT_DAEMON);

    return isdaemon;
}

int conf_set_daemon(int d)
{
    /* should check if gophrier is running */
    isdaemon = d;

    return 0;
}

int conf_use_ipv6()
{
    if (useipv6 == -1)
        useipv6 = conf_read_int("ipv6", GOPHRIER_DEFAULT_IPV6);

    return useipv6;
}

int conf_set_ipv6(int i)
{
    useipv6 = i;

    return 0;
}

int conf_get_port()
{
    if (port == -1)
        port = conf_read_int("port", GOPHRIER_DEFAULT_PORT);

    return port;
}

int conf_set_port(int p)
{
    /* should check if gophrier is running */
    port = p;

    return 0;
}

static int conf_read_int(const char * confkey, int defaultvalue)
{
    char buffer[1024];
    FILE * conffile;
    int value;
    
    conffile = conf_open(confkey);
    if (! conffile) return defaultvalue;

    if (NULL == fgets(buffer, 1024, conffile)) return defaultvalue;
    value = strtol(buffer, NULL, 10);
    fclose(conffile);
    return value;
}

static char * conf_read_string(const char * confkey, const char * defaultvalue)
{
    char buffer[1024];
    FILE * conffile;
    char * value;
    
    conffile = conf_open(confkey);
    if (! conffile) return strdup(defaultvalue);
    
    if (NULL == fgets(buffer, 1024, conffile)) return strdup(defaultvalue);
    {
        char * fixnl;
        fixnl = strchr(buffer, '\n');
        if (fixnl)
            *fixnl = 0;
    }
    value = strdup(buffer);
    fclose(conffile);
    return value;
}

static FILE * conf_open(const char * conf)
{
    char * fullconf;
    char * confdir;
    FILE * file;
    
    confdir = conf_get_conf_dir();
    
    fullconf = (char *) malloc(strlen(confdir) + 1 + strlen(conf) + 1);
    sprintf(fullconf, "%s/%s", confdir, conf);
    file = fopen(fullconf, "r");
    free(fullconf);

    return file;
}
